/*
** File: device.h  
** Project: ADMXRC3 module driver
** Purpose: Data structures and definitions private to ADMXRC3 module driver,
**          including the device context structure.
**
** (C) Copyright Alpha Data 2009-2011
*/

#if !defined(ADATA_ADMXRC3_DEVICE_H)
#define ADATA_ADMXRC3_DEVICE_H

#include <df.h>

#include <core/coreif.h>
#include <core/shared_defs.h>
#include "device_defs.h"

typedef int32_t   _ADMXRC3_INT32;
typedef uint64_t  _ADMXRC3_UINT64;
typedef uint32_t  _ADMXRC3_UINT32;
typedef uint16_t  _ADMXRC3_UINT16;
typedef uint8_t   _ADMXRC3_UINT8;
typedef uint8_t   _ADMXRC3_BYTE;
typedef boolean_t _ADMXRC3_BOOL;

#if DF_HAS_USER_EVENT_HANDLE
# define ADMXRC3_HAS_USER_EVENT_HANDLE (1)
typedef user_event_handle_t admxrc3_user_event_handle_t;
# if DF_NEED_THUNK
typedef user_event_handle_t_32 admxrc3_user_event_handle_t_32;
# endif
#else
# define ADMXRC3_HAS_USER_EVENT_HANDLE (0)
#endif
#if DF_NEED_THUNK
# define ADMXRC3_NEED_THUNK (1)
#endif
#include <ioctl_admxrc3.h>
#include <admxrc3/status.h>

struct _BufferDescription;
typedef struct _BufferDescription BufferDescription;
struct _Admxrc3DeviceContext;
typedef struct _Admxrc3DeviceContext Admxrc3DeviceContext;
struct _DmaRequestContext;
typedef struct _DmaRequestContext DmaRequestContext;
struct _Admxrc3ClientContext;
typedef struct _Admxrc3ClientContext Admxrc3ClientContext;
struct _ClockGeneratorTicket;
typedef struct _ClockGeneratorTicket ClockGeneratorTicket;
struct _DmaTicket;
typedef struct _DmaTicket DmaTicket;
struct _FpgaAlertNotification;
typedef struct _FpgaAlertNotification FpgaAlertNotification;
struct _FpgaInterruptNotification;
typedef struct _FpgaInterruptNotification FpgaInterruptNotification;
struct _GenericRequestContext;
typedef struct _GenericRequestContext GenericRequestContext;
struct _NotificationWaitList;
typedef struct _NotificationWaitList NotificationWaitList;
struct _NotificationTicket;
typedef struct _NotificationTicket NotificationTicket;
struct _SensorPollNotification;
typedef struct _SensorPollNotification SensorPollNotification;
struct _UserBufferMappings;
typedef struct _UserBufferMappings UserBufferMappings;
#if DF_HAS_USER_EVENT_HANDLE
struct _UserEventInfo;
typedef struct _UserEventInfo UserEventInfo;
struct _UserEventList;
typedef struct _UserEventList UserEventList;
#endif
struct _RequestContext;
typedef struct _RequestContext RequestContext;

#if DF_HAS_USER_EVENT_HANDLE
struct _UserEventInfo {
  Admxrc3ClientContext* pClCtx;
  user_event_handle_t hUserEvent;
  DfUserEvent userEvent;
  UserEventInfo* pPrev;
  UserEventInfo* pNext;
};

struct _UserEventList {
  DfSpinLock lock;
  UserEventInfo* pHead;
  UserEventInfo* pTail;
};
#endif

struct _ClockGeneratorTicket {
  CoreTicket ticket;
  CoreClockStatus status;
  DfEvent event;
};

struct _NotificationWaitList {
  DfSpinLock lock;
  DfListNode header;
};

struct _DmaTicket {
  CoreTicket ticket;
  DfSpinLock lock;
  CoreDmaParameters dmaParameters;
  unsigned int dmaChannel;
	Admxrc3DeviceContext* pDevCtx;
	Admxrc3ClientContext* pClCtx;
  DfClientRequest* pRequest;
  boolean_t bDmaLocked;
  union {
    BufferDescription* pBuffer;        /* Used by ioctl{Read|Write}DmaLocked */
    DfBufferDescription* pDescription; /* Used by ioctl{Read|Write}Dma */
  } variant;
  unsigned int state;
  struct {
    boolean_t bCancel;
    DfIoStatus reason;
  } cancel;
};

struct _NotificationTicket {
  DfListNode node;
	Admxrc3DeviceContext* pDevCtx;
  DfClientRequest* pRequest;
  unsigned int state;
  unsigned int notification;
};

/* Structure that keeps track of mappings between ADMXRC3_BUFFER_HANDLE and user space buffers */
struct _UserBufferMappings {
  DfSpinLock lock;
  /* Maps a buffer handle (uint32_t) to a BufferDescription* */
  BufferDescription* map[MAX_NUM_LOCKED_USER_SPACE_BUFFER];
  /* Stack that contains free handles */
  struct {
    uint32_t free[MAX_NUM_LOCKED_USER_SPACE_BUFFER];
    unsigned int top; /* Stack top pointer */
  } stack;
};

struct _Admxrc3ClientContext {
	DfDeviceObject* pDevObj;
  Admxrc3DeviceContext* pDevCtx;
  boolean_t bPrivileged;

#if !ADMXRC3_LOCKED_BUFFERS_GLOBAL
  /* Information about locked user-space buffers */
  UserBufferMappings lockedBuffers;
#endif

#if ADMXRC3_SINGLE_REQUEST_PER_HANDLE
  /*
  ** If ADMXRC_SINGLE_REQUEST_PER_HANDLE is defined, then a restriction is enforced
  ** whereby at most one I/O request can be ongoing per open device handle.
  */
  struct {
    DfRequest* pRequest; /* Current I/O request */
    union {              /* Context for current I/O request */
      ClockGeneratorTicket clockGen;
      DmaTicket dma;
    } context;
  } currentRequest;
#endif
};

struct _FpgaAlertNotification {
  CoreNotification coreNotification;
  unsigned int targetIndex;
};

struct _FpgaInterruptNotification {
  CoreNotification coreNotification;
  unsigned int targetIndex;
};

struct _SensorPollNotification {
  CoreNotification coreNotification;
};

struct _Admxrc3DeviceContext {
  DfDeviceObject* pDevObj;
  Adb3CoreInterface coreInterface;
  void* pCoreContext;
  boolean_t bCoreInterfaceImpd;
  boolean_t bReverseSelectMapData;
  uint64_t localAddressLimit;
  uint8_t localWidth;

  struct {
    DfInterface iface;
    boolean_t bEnabled;
    boolean_t bRegistered;
  } userMode;

  struct {
    unsigned int numFlashBank;
    unsigned int numTargetFpga;
    unsigned int numWindow;
  } info;

  struct {
    ADMXRC3_STATUS (*pGetBankInfo)(Admxrc3DeviceContext* pDevCtx, unsigned int bankIndex, const void* pVpdBuffer, IOCTLS_ADMXRC3_GETBANKINFO* pIoctl);
    void           (*pGetCardInfo)(Admxrc3DeviceContext* pDevCtx, const void* pVpdBuffer, IOCTLS_ADMXRC3_GETCARDINFO* pIoctl);
    ADMXRC3_STATUS (*pGetFpgaInfo)(Admxrc3DeviceContext* pDevCtx, unsigned int targetIndex, const void* pVpdBuffer, IOCTLS_ADMXRC3_GETFPGAINFO* pIoctl);
  } methods;

  struct {
    boolean_t bTicketPoolValid;
    DfPool ticketPool;
  } dma;

  struct {
    boolean_t bTicketPoolValid;
    DfPool ticketPool;
  } clockGenerator;

  struct {
    struct {
      DfSpinLock lock;
      Admxrc3ClientContext* pClCtx;
    } owner;
    struct {
      FpgaInterruptNotification notification;
      boolean_t bNotificationRegistered;
#if DF_HAS_USER_EVENT_HANDLE 
      UserEventList eventList;
#endif
      NotificationWaitList waitList;
    } interrupt;
    struct {
      FpgaAlertNotification notification;
      boolean_t bNotificationRegistered;
#if DF_HAS_USER_EVENT_HANDLE 
      UserEventList eventList;
#endif
      NotificationWaitList waitList;
    } alert;
  } fpga[MAX_NUM_TARGET_FPGA];

  struct {
    uint8_t* pStagingBuffer;
  } flash[MAX_NUM_FLASH_BANK];

  struct {
    boolean_t bTicketPoolValid;
    DfPool ticketPool;
  } notification;

  struct {
      SensorPollNotification notification;
      boolean_t bNotificationRegistered;
  } sensor;
};

struct _BufferDescription {
  uint32_t handle;
  uint32_t refCount;
  Admxrc3ClientContext* pCreator;
  DfBufferDescription* pDfBufferDesc;
};

#endif
