/*
** File: identify.c  
** Project: ADMXRC3 module driver
** Purpose: Identify device given model, bootstrap the the device context
**          appropriately for that model.
**
** (C) Copyright Alpha Data 2010
*/

#include <df.h>

#include "device.h"
#include "identify.h"
#include "model_boot.h"

boolean_t
admxrc3Identify(
  Admxrc3DeviceContext* pDevCtx,
  CoreModelType model)
{
#if defined(ADMXRC3_CUSTOM)
  if (admxrc3IdentifyCustom(pDevCtx, model)) {
    return TRUE; /* Success */
  }
#endif

#if !defined(ADMXRC3_CUSTOM_ONLY)
  switch (model) {
    case CoreModelGeneric:
      admxrc3BootstrapGeneric(pDevCtx);
      return TRUE;

    case CoreModelAdmxrc2:
      admxrc3BootstrapAdmxrc2(pDevCtx);
      return TRUE;

    case CoreModelAdpexrc5t:
      admxrc3BootstrapAdpexrc5t(pDevCtx);
      return TRUE;

    case CoreModelAdmxrc6tl:
      admxrc3BootstrapAdmxrc6tl(pDevCtx);
      return TRUE;

    case CoreModelAdmxrc6t1:
    case CoreModelAdmxrc6t1Bert:
      admxrc3BootstrapAdmxrc6t1(pDevCtx);
      return TRUE;

    default:
      break;
  }
#endif

	return FALSE; /* Failure */
}
